# ------------------------------------------
# | RepeatTimer by Ben Corne               |
# | Contact: ben.corne(-at-)gmail.com      |
# | Version: 0.1                           |
# | Date Modified: 13 June 2010            |
# ------------------------------------------

# Info:
# -----
# This package provides a class RepeatTimer which provides an
# API to start and stop an action that gets repeated each x
# seconds.
# It provides factory methods for a timer which gets repeated
# forever and one for a timer that runs a finite nr of times.

import threading

class RepeatTimer:

	def __init__(self, fun, every, times):
		self.repeat = times
		self.function = fun	
		self.every = every
		self.timer = 0
		self.__lock = threading.Lock()

	@classmethod
	def newFinal(self, fun, every, times):
		return RepeatTimer(fun,every,times)
	
	@classmethod
	def newEternal(self, fun, every):
		return RepeatTimer(fun,every,-1)
	
	def __loop(self):
		if self.repeat == 0:
			return # stop if 0, if < 0 just keep going forever
		self.function()
		self.repeat = self.repeat - 1
		self.__setTimer(threading.Timer(self.every,self.__loop))
		self.__getTimer().start()
	
	def __setTimer(self,t):
		self.__lock.acquire()
		self.timer = t
		self.__lock.release()
	
	def __getTimer(self):
		return self.timer
	
	def start(self):
		self.__loop()

	def kill(self):
		self.__lock.acquire()
		self.timer.cancel()
		self.__lock.release()

# Example use
# -----------
# def hello():
# 	print "Hello"
# 
# #-- Eternal Repeating Timer
# timer = RepeatTimer.newEternal(hello,3)
# timer.start()
# threading.Timer(4,lambda:timer.kill()).start()
#
# #-- Final Repeating Timer
# timer = RepeatTimer.newFinal(hello,3,4)
# timer.start()
# threading.Timer(10,lambda:timer.kill()).start()

# TODO's
# ------
# 1] Allow to choose to call the repeating function at the
#    start or at the end of the timer rounds
